import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:vengamo_chat_ui/components/place_holder.dart';
import 'package:vengamo_chat_ui/theme/app_color.dart';
import '../Utils/index.dart';
import '../seen_status.dart';
import 'image_with_loading_indicator.dart';

class ImageWithCaption extends StatelessWidget {
  const ImageWithCaption(
      {super.key,
      required this.imgUrl,
      required this.caption,
      required this.constraints,
      required this.color,
      required this.isMe,
      required this.ack,
      required this.time,
      required this.timeLabelColor,
      required this.showLoadingOverlay,
      required this.percentage,
      required this.onTap,
      this.textStyle,
      this.imageStyles,
      this.messageReplyBgColor,
      this.messageReplyBorderColor,
      required this.messageReply});

  /// Represents the URL of an image.
  final String? imgUrl;

  /// Represents the caption or textual description associated with an image.
  final Widget? caption;

  /// Represents the constraints applied to a widget's size.
  ///
  /// The [constraints] parameter is used to define the maximum width and height that a widget can occupy.
  /// It is commonly used in layout calculations to determine the size and positioning of child widgets within a parent widget.
  ///
  /// Example usage:
  /// dart /// Container( /// constraints: BoxConstraints( /// maxWidth: constraints.maxWidth * 0.8, /// ), /// child: ... /// ) /// ///
  /// In this example, the maxWidth property of the constraints is set to 80% of the maximum width available to the parent widget.
  /// This constraint ensures that the child widget, wrapped inside the Container, does not exceed the specified maximum width.

  final BoxConstraints constraints;

  ///Background Color
  final Color color;

  /// this indicates if the message is from me or the other user
  final bool isMe;

  /// Message status example ( sent, delivered and seen)
  final Widget ack;

  /// message time
  final String time;

  /// color that will be applied on time label.
  final Color timeLabelColor;

  /// Add this boolean variable to control the visibility of the loading overlay
  final bool showLoadingOverlay;

  /// The `percentage` represents the progress of the image upload or download operation.
  /// It indicates the completion percentage of the image transfer process,
  /// providing feedback to the user about the progress of the operation.
  /// The value of `percentage` ranges from 0.0 to 1.0, where 0.0 represents no progress,
  /// and 1.0 represents the completion of the operation.
  final double percentage;

  //call back function that will be fired on tap.
  final VoidCallback onTap;
  // text style to be applied on caption
  final TextStyle? textStyle;
  // styles that will be applies on Image.
  final BoxConstraints? imageStyles;

  final Color? messageReplyBgColor;

  /// the Border color applied to the message reply
  final Color? messageReplyBorderColor;

  /// message Reply
  final Widget? messageReply;

  @override
  Widget build(BuildContext context) {
    if (imgUrl == "" || imgUrl == null) {
      return Container();
    }
    //validate if image url sent is indeed base64
    Map<String, dynamic> isNormalUrlOrPath = urlOrPathOrBase64(imgUrl!);

    if (isNormalUrlOrPath['isBase64String']) {
      return Container(
        constraints: BoxConstraints(
          minWidth: (adjustMinWidth(imageStyles?.minWidth)) - 47,
          maxWidth: (adjustMaxWidth(imageStyles?.maxWidth)) - 47,
          minHeight: (adjustMinHeight(imageStyles?.minHeight)) - 47,
          maxHeight: (adjustMaxHeight(imageStyles?.maxHeight)) - 47,
        ),
        decoration: const BoxDecoration(
          borderRadius: BorderRadius.all(Radius.circular(9)),
        ),
        child: const PlaceHolder(),
      );
    } else {
      return Padding(
        padding: const EdgeInsets.symmetric(horizontal: 2),
        child: Container(
          decoration: BoxDecoration(
            color: color,
            borderRadius: const BorderRadius.all(Radius.circular(9)),
          ),
          constraints: messageReply != null
              ? BoxConstraints(
                  maxWidth: constraints.maxWidth * 0.8,
                  minHeight: 10,
                )
              : BoxConstraints(
                  maxWidth: constraints.maxWidth * 0.8,
                  minHeight: adjustMinHeight(imageStyles?.minHeight),
                ),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              ClipRRect(
                borderRadius: BorderRadius.circular(9),
                child: Stack(
                  children: [
                    InkWell(
                      onTap: onTap,
                      child: Padding(
                        padding: messageReply != null
                            ? const EdgeInsets.all(4.0)
                            : const EdgeInsets.all(3.0),
                        child: isNormalUrlOrPath['isFilePath']
                            ? ImageWithLoadingIndicator(
                                imagePath: imgUrl!,
                                constraints: constraints,
                                imageStyles: imageStyles)
                            : messageReply != null
                                ? Column(
                                    mainAxisAlignment: MainAxisAlignment.start,
                                    crossAxisAlignment:
                                        CrossAxisAlignment.start,
                                    children: [
                                      Stack(
                                        children: [
                                          Container(
                                            decoration: BoxDecoration(
                                                color: messageReplyBgColor ??
                                                    (isMe
                                                        ? const Color(
                                                                0xFFB3D19D)
                                                            .withOpacity(0.5)
                                                        : Colors.grey[300]),
                                                borderRadius:
                                                    const BorderRadius.only(
                                                  topLeft: Radius.circular(3),
                                                  bottomLeft:
                                                      Radius.circular(3),
                                                )),
                                            constraints: const BoxConstraints(
                                                maxHeight: 60),
                                            child: Padding(
                                              padding: const EdgeInsets.only(
                                                  left: 10,
                                                  top: 3,
                                                  bottom: 2,
                                                  right: 2),
                                              child: Row(
                                                crossAxisAlignment:
                                                    CrossAxisAlignment.start,
                                                mainAxisAlignment:
                                                    MainAxisAlignment
                                                        .spaceBetween,
                                                children: [
                                                  Column(
                                                    crossAxisAlignment:
                                                        CrossAxisAlignment
                                                            .start,
                                                    children: [
                                                      Text(
                                                          isMe
                                                              ? "Sent"
                                                              : "Received",
                                                          style:
                                                              textStyle), // "Sent" text
                                                      const SizedBox(
                                                          height:
                                                              8), // Add space between "Sent" and photo icon
                                                      const Icon(
                                                        Icons
                                                            .camera_alt_rounded,
                                                        color:
                                                            AppColors.iconColor,
                                                      ), // Photo icon
                                                    ],
                                                  ),
                                                  const SizedBox(
                                                      width:
                                                          8), // Add space between photo icon and image

                                                  SizedBox(
                                                    height: 90,
                                                    width: 60,
                                                    child: CachedNetworkImage(
                                                        imageUrl: imgUrl!,
                                                        imageBuilder: (context,
                                                                imageProvider) =>
                                                            ClipRRect(
                                                              borderRadius:
                                                                  BorderRadius
                                                                      .circular(
                                                                          2),
                                                              child: Container(
                                                                decoration:
                                                                    BoxDecoration(
                                                                  image:
                                                                      DecorationImage(
                                                                    image:
                                                                        imageProvider,
                                                                    fit: BoxFit
                                                                        .cover,
                                                                    colorFilter: const ColorFilter
                                                                            .mode(
                                                                        Colors
                                                                            .black12,
                                                                        BlendMode
                                                                            .colorBurn),
                                                                  ),
                                                                ),
                                                              ),
                                                            ),
                                                        placeholder: (context,
                                                                url) =>
                                                            const PlaceHolder(),
                                                        errorWidget: (context,
                                                            url, error) {
                                                          return const Center(
                                                            child: Icon(
                                                                Icons.error),
                                                          );
                                                        }),
                                                  )
                                                  // The actual photo on the right
                                                ],
                                              ),
                                            ),
                                          ),
                                          Positioned(
                                            top: 0,
                                            bottom: 0,
                                            left:
                                                1, // Adjust the value as needed for your design
                                            child: Container(
                                              decoration: BoxDecoration(
                                                color:
                                                    messageReplyBorderColor ??
                                                        (isMe
                                                            ? const Color(
                                                                0xFF2BCA72)
                                                            : Colors.grey),
                                                borderRadius:
                                                    const BorderRadius.only(
                                                  topLeft: Radius.circular(25),
                                                  bottomLeft:
                                                      Radius.circular(25),
                                                ),
                                              ),
                                              width:
                                                  4, // Border width// Border color
                                            ),
                                          ),
                                        ],
                                      ),
                                      messageReply ?? Container()
                                    ],
                                  )
                                : Container(
                                    constraints: BoxConstraints(
                                      minWidth: (adjustMinWidth(
                                              imageStyles?.minWidth)) -
                                          47,
                                      maxWidth: (adjustMaxWidth(
                                              imageStyles?.maxWidth)) -
                                          47,
                                      minHeight: (adjustMinHeight(
                                              imageStyles?.minHeight)) -
                                          47,
                                      maxHeight: (adjustMaxHeight(
                                              imageStyles?.maxHeight)) -
                                          47,
                                    ),
                                    decoration: const BoxDecoration(
                                      borderRadius:
                                          BorderRadius.all(Radius.circular(9)),
                                    ),
                                    child: CachedNetworkImage(
                                        imageUrl: imgUrl!,
                                        imageBuilder: (context,
                                                imageProvider) =>
                                            ClipRRect(
                                              borderRadius:
                                                  BorderRadius.circular(9),
                                              child: Container(
                                                decoration: BoxDecoration(
                                                  image: DecorationImage(
                                                    image: imageProvider,
                                                    fit: BoxFit.cover,
                                                    colorFilter:
                                                        const ColorFilter.mode(
                                                            Colors.black12,
                                                            BlendMode
                                                                .colorBurn),
                                                  ),
                                                ),
                                              ),
                                            ),
                                        placeholder: (context, url) =>
                                            const PlaceHolder(),
                                        errorWidget: (context, url, error) {
                                          return const Center(
                                            child: Icon(Icons.error),
                                          );
                                        }),
                                  ),
                      ),
                    ),
                    if (showLoadingOverlay) // Display the loading overlay conditionally based on the showLoadingOverlay variable
                      Positioned.fill(
                        child: Padding(
                          padding: const EdgeInsets.all(3.0),
                          child: Container(
                            decoration: BoxDecoration(
                              borderRadius:
                                  const BorderRadius.all(Radius.circular(9)),
                              color: Colors.black.withOpacity(0.7),
                            ),
                            child: Center(
                              child: Container(
                                width: 60,
                                height: 60,
                                decoration: BoxDecoration(
                                  shape: BoxShape.circle,
                                  color: Colors.black.withOpacity(0.8),
                                ),
                                child: Stack(
                                  children: [
                                    const Center(
                                      child: Icon(Icons.close,
                                          color: Colors.white, size: 32),
                                    ),
                                    Positioned.fill(
                                      child: CircularProgressIndicator(
                                        valueColor:
                                            const AlwaysStoppedAnimation<Color>(
                                                Colors.white),
                                        value:
                                            percentage, // Replace "progress" with the actual progress value (0.0 to 1.0)
                                        strokeWidth: 2,
                                      ),
                                    ),
                                  ],
                                ),
                              ),
                            ),
                          ),
                        ),
                      ),
                  ],
                ),
              ),
              Container(
                padding: const EdgeInsets.all(6),
                decoration: const BoxDecoration(
                  borderRadius: BorderRadius.only(
                    bottomLeft: Radius.circular(9),
                    bottomRight: Radius.circular(9),
                  ),
                ),
                child: Row(
                  crossAxisAlignment: CrossAxisAlignment.end,
                  children: [
                    Expanded(
                      child: Padding(
                        padding: const EdgeInsets.only(top: 2.0, bottom: 4.0),
                        child: messageReply == null ? caption : Container(),
                      ),
                    ),
                    Align(
                      alignment: Alignment.bottomRight,
                      child: Padding(
                        padding: const EdgeInsets.only(bottom: 0.0),
                        child: SeenStatus(
                          isMe: isMe,
                          time: time,
                          dateTextColor: timeLabelColor,
                          messageAck: ack,
                        ),
                      ),
                    ),
                  ],
                ),
              )
            ],
          ),
        ),
      );
    }
  }
}
